@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2008-2014 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 Kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@ import webbrowser
 import logging
 from signal import signal, SIGABRT, SIGINT, SIGTERM
 
-from log import logError, logDebug, m18n, m18nc
+from log import logError, logDebug
 from common import Options, Internal, isAlive, Debug
 
 class MyHook(cgitb.Hook):
@@ -60,38 +60,45 @@ class MyHook(cgitb.Hook):
 NOTFOUND = []
 
 try:
-    from qt import Qt, toQVariant, variantValue, QEvent, QMetaObject, PYQT_VERSION_STR, QTimer
+    from qt import Qt, QEvent, QMetaObject, QTimer
     from qt import QWidget, QGridLayout, QAction
 except ImportError as importError:
-    NOTFOUND.append('Please install PyQt4 or PyQt5: %s' % importError)
+    NOTFOUND.append('Please install PyQt5: %s' % importError)
 
 try:
     from zope.interface import implements  # pylint: disable=unused-import
 except ImportError as importError:
     NOTFOUND.append('Package python-zope-interface missing: %s' % importError)
 
-from kde import KIcon, KAction, KApplication, KToggleFullScreenAction, \
-    KXmlGuiWindow, KStandardAction
+try:
+    from twisted.spread import pb # pylint: disable=unused-import
+    from twisted.internet.error import ReactorNotRunning
+except ImportError as importError:
+    NOTFOUND.append('Package python3-twisted is missing or too old (I need 16.6.0): %s' % importError)
 
-from board import FittingView
-from playerlist import PlayerList
-from tileset import Tileset
-from background import Background
-from scoring import scoreGame
-from scoringdialog import ScoreTable, ExplainView
-from humanclient import HumanClient
-from rulesetselector import RulesetSelector
-from animation import afterQueuedAnimations, MoveImmediate
-from chat import ChatWindow
-from scene import PlayingScene, ScoringScene
-from configdialog import ConfigDialog
-from statesaver import StateSaver
-from util import checkMemory
-from twisted.internet.error import ReactorNotRunning
 
-# except ImportError as importError:
-# NOTFOUND.append('Kajongg is not correctly installed: modules: %s' %
-# importError)
+try:
+    from mi18n import i18n, i18nc
+    from kde import KIcon, KXmlGuiWindow, KStandardAction
+
+    from board import FittingView
+    from playerlist import PlayerList
+    from tileset import Tileset
+    from background import Background
+    from scoring import scoreGame
+    from scoringdialog import ScoreTable, ExplainView
+    from humanclient import HumanClient
+    from rulesetselector import RulesetSelector
+    from animation import afterQueuedAnimations, AnimationSpeed, ParallelAnimationGroup
+    from chat import ChatWindow
+    from scene import PlayingScene, ScoringScene
+    from configdialog import ConfigDialog
+    from statesaver import StateSaver
+    from util import checkMemory
+
+except ImportError as importError:
+    NOTFOUND.append('Kajongg is not correctly installed: modules: %s' %
+                    importError)
 
 if len(NOTFOUND):
     logError("\n".join(" * %s" % s for s in NOTFOUND), showStack=False)
@@ -102,12 +109,12 @@ def cleanExit(*dummyArgs):
     """close sqlite3 files before quitting"""
     if isAlive(Internal.mainWindow):
         if Debug.quit:
-            logDebug(u'cleanExit calling mainWindow.close')
+            logDebug('cleanExit calling mainWindow.close')
         Internal.mainWindow.close()
     else:
         # this must be very early or very late
         if Debug.quit:
-            logDebug(u'cleanExit calling sys.exit(0)')
+            logDebug('cleanExit calling sys.exit(0)')
         # sys.exit(0)
         MainWindow.aboutToQuit()
 
@@ -139,6 +146,7 @@ class MainWindow(KXmlGuiWindow):
         self.playerWindow = None
         self.rulesetWindow = None
         self.confDialog = None
+        self.__installReactor()
         if Options.gui:
             KStandardAction.preferences(
                 self.showSettings,
@@ -163,6 +171,18 @@ class MainWindow(KXmlGuiWindow):
         else:
             HumanClient()
 
+    @staticmethod
+    def __installReactor():
+        """install the twisted reactor"""
+        if Internal.reactor is None:
+            import qtreactor
+            qtreactor.install()
+            from twisted.internet import reactor
+            reactor.runReturn(installSignalHandlers=False)
+            Internal.reactor = reactor
+            if Debug.quit:
+                logDebug('Installed qtreactor')
+
     @property
     def scene(self):
         """a proxy"""
@@ -179,11 +199,11 @@ class MainWindow(KXmlGuiWindow):
             self.actionChat.setChecked(False)
             self.actionExplain.setChecked(False)
             self.actionScoreTable.setChecked(False)
-            self.actionExplain.setData(toQVariant(ExplainView))
-            self.actionScoreTable.setData(toQVariant(ScoreTable))
+            self.actionExplain.setData(ExplainView)
+            self.actionScoreTable.setData(ScoreTable)
         self._scene = value
         self.centralView.setScene(value)
-        self.adjustView()
+        self.adjustMainView()
         self.updateGUI()
         self.actionChat.setEnabled(isinstance(value, PlayingScene))
         self.actionExplain.setEnabled(value is not None)
@@ -196,7 +216,7 @@ class MainWindow(KXmlGuiWindow):
                         # we want space to the right for the buttons
         # the default is too small. Use at least 2/3 of screen height and 1/2
         # of screen width:
-        available = KApplication.kApplication().desktop().availableGeometry()
+        available = Internal.app.desktop().availableGeometry()
         height = max(result.height(), available.height() * 2 // 3)
         width = max(result.width(), available.width() // 2)
         result.setHeight(height)
@@ -211,7 +231,7 @@ class MainWindow(KXmlGuiWindow):
     def kajonggAction(
             self, name, icon, slot=None, shortcut=None, actionData=None):
         """simplify defining actions"""
-        res = KAction(self)
+        res = QAction(self)
         if icon:
             res.setIcon(KIcon(icon))
         if slot:
@@ -220,8 +240,8 @@ class MainWindow(KXmlGuiWindow):
         if shortcut:
             res.setShortcut(Qt.CTRL + shortcut)
             res.setShortcutContext(Qt.ApplicationShortcut)
-        if PYQT_VERSION_STR != '4.5.2' or actionData is not None:
-            res.setData(toQVariant(actionData))
+        if actionData is not None:
+            res.setData(actionData)
         return res
 
     def _kajonggToggleAction(self, name, icon, shortcut=None, actionData=None):
@@ -232,7 +252,8 @@ class MainWindow(KXmlGuiWindow):
             shortcut=shortcut,
             actionData=actionData)
         res.setCheckable(True)
-        res.toggled.connect(self._toggleWidget)
+        if actionData is not None:
+            res.toggled.connect(self._toggleWidget)
         return res
 
     def setupUi(self):
@@ -254,7 +275,7 @@ class MainWindow(KXmlGuiWindow):
         self.centralView.setFocusPolicy(Qt.StrongFocus)
         self.background = None  # just for pylint
         self.windTileset = Tileset(Internal.Preferences.windTilesetName)
-        self.adjustView()
+        self.adjustMainView()
         self.actionScoreGame = self.kajonggAction(
             "scoreGame",
             "draw-freehand",
@@ -263,7 +284,7 @@ class MainWindow(KXmlGuiWindow):
         self.actionPlayGame = self.kajonggAction(
             "play",
             "arrow-right",
-            self.playingScene,
+            self.playGame,
             Qt.Key_N)
         self.actionAbortGame = self.kajonggAction(
             "abort",
@@ -291,13 +312,6 @@ class MainWindow(KXmlGuiWindow):
             self.changeAngle,
             Qt.Key_G)
         self.actionAngle.setEnabled(False)
-        self.actionFullscreen = KToggleFullScreenAction(
-            self.actionCollection())
-        self.actionFullscreen.setShortcut(Qt.CTRL + Qt.Key_F)
-        self.actionFullscreen.setShortcutContext(Qt.ApplicationShortcut)
-        self.actionFullscreen.setWindow(self)
-        self.actionCollection().addAction("fullscreen", self.actionFullscreen)
-        self.actionFullscreen.toggled.connect(self.fullScreen)
         self.actionScoreTable = self._kajonggToggleAction(
             "scoreTable", "format-list-ordered",
             Qt.Key_T, actionData=ScoreTable)
@@ -306,6 +320,9 @@ class MainWindow(KXmlGuiWindow):
             "explain", "applications-education",
             Qt.Key_E, actionData=ExplainView)
         self.actionExplain.setEnabled(False)
+        self.actionFullscreen = self._kajonggToggleAction(
+            "fullscreen", "view-fullscreen", shortcut=Qt.Key_F + Qt.ShiftModifier)
+        self.actionFullscreen.toggled.connect(self.fullScreen)
         self.actionAutoPlay = self.kajonggAction(
             "demoMode",
             "arrow-right-double",
@@ -317,6 +334,12 @@ class MainWindow(KXmlGuiWindow):
         self.actionAutoPlay.setChecked(Internal.autoPlay)
         QMetaObject.connectSlotsByName(self)
 
+    def playGame(self):
+        """manual wish for a new game"""
+        if not Internal.autoPlay:
+            # only if no demo game is running
+            self.playingScene()
+
     def playingScene(self):
         """play a computer game: log into a server and show its tables"""
         self.scene = PlayingScene(self)
@@ -334,11 +357,15 @@ class MainWindow(KXmlGuiWindow):
 
     def fullScreen(self, toggle):
         """toggle between full screen and normal view"""
-        self.actionFullscreen.setFullScreen(self, toggle)
+        if toggle:
+            self.setWindowState(self.windowState() | Qt.WindowFullScreen)
+        else:
+            self.setWindowState(self.windowState() & ~Qt.WindowFullScreen)
 
     def close(self, dummyResult=None):
         """wrap close() because we call it with a QTimer"""
         if isAlive(self):
+            ParallelAnimationGroup.cancelAll()
             return KXmlGuiWindow.close(self)
 
     def closeEvent(self, event):
@@ -383,9 +410,9 @@ class MainWindow(KXmlGuiWindow):
             self.exitConfirmed = bool(result)
             if Debug.quit:
                 if self.exitConfirmed:
-                    logDebug(u'mainWindow.queryClose confirmed')
+                    logDebug('mainWindow.queryClose confirmed')
                 else:
-                    logDebug(u'mainWindow.queryClose not confirmed')
+                    logDebug('mainWindow.queryClose not confirmed')
             # start closing again. This time no question will appear, the game
             # is already aborted
             if self.exitConfirmed:
@@ -397,7 +424,7 @@ class MainWindow(KXmlGuiWindow):
         def cancelled(result):
             """just do nothing"""
             if Debug.quit:
-                logDebug(u'mainWindow.queryClose.cancelled: {}'.format(result))
+                logDebug('mainWindow.queryClose.cancelled: {}'.format(result))
             self.exitConfirmed = None
         if self.exitConfirmed is False:
             # user is currently being asked
@@ -410,7 +437,7 @@ class MainWindow(KXmlGuiWindow):
                 self.exitConfirmed = True
                 if Debug.quit:
                     logDebug(
-                        u'MainWindow.queryClose not asking, exitConfirmed=True')
+                        'MainWindow.queryClose not asking, exitConfirmed=True')
         return True
 
     def queryExit(self):
@@ -421,7 +448,7 @@ class MainWindow(KXmlGuiWindow):
                 logDebug(*args, **kwargs)
 
         if self.exitReady:
-            quitDebug(u'MainWindow.queryExit returns True because exitReady is set')
+            quitDebug('MainWindow.queryExit returns True because exitReady is set')
             return True
         if self.exitConfirmed:
             # now we can get serious
@@ -438,20 +465,20 @@ class MainWindow(KXmlGuiWindow):
                 self.exitWaitTime += 10
                 if self.exitWaitTime % 1000 == 0:
                     logDebug(
-                        u'waiting since %d seconds for reactor to stop' %
+                        'waiting since %d seconds for reactor to stop' %
                         (self.exitWaitTime // 1000))
                 try:
-                    quitDebug(u'now stopping reactor')
+                    quitDebug('now stopping reactor')
                     Internal.reactor.stop()
                     assert isAlive(self)
                     QTimer.singleShot(10, self.close)
                 except ReactorNotRunning:
                     self.exitReady = True
                     quitDebug(
-                        u'MainWindow.queryExit returns True: It got exception ReactorNotRunning')
+                        'MainWindow.queryExit returns True: It got exception ReactorNotRunning')
             else:
                 self.exitReady = True
-                quitDebug(u'MainWindow.queryExit returns True: Reactor is not running')
+                quitDebug('MainWindow.queryExit returns True: Reactor is not running')
         return bool(self.exitReady)
 
     @staticmethod
@@ -461,14 +488,14 @@ class MainWindow(KXmlGuiWindow):
         Internal.mainWindow = None
         if mainWindow:
             if Debug.quit:
-                logDebug(u'aboutToQuit starting')
+                logDebug('aboutToQuit starting')
             if mainWindow.exitWaitTime > 1000.0 or Debug.quit:
                 logDebug(
-                    u'reactor stopped after %d ms' %
+                    'reactor stopped after %d ms' %
                     (mainWindow.exitWaitTime))
         else:
             if Debug.quit:
-                logDebug(u'aboutToQuit: mainWindow is already None')
+                logDebug('aboutToQuit: mainWindow is already None')
         StateSaver.saveAll()
         Internal.app.quit()
         try:
@@ -481,76 +508,80 @@ class MainWindow(KXmlGuiWindow):
         checkMemory()
         logging.shutdown()
         if Debug.quit:
-            logDebug(u'aboutToQuit ending')
+            logDebug('aboutToQuit ending')
 
     def abortAction(self):
         """abort current game"""
         if Debug.quit:
-            logDebug(u'mainWindow.abortAction invoked')
+            logDebug('mainWindow.abortAction invoked')
         return self.scene.abort()
 
     def retranslateUi(self):
         """retranslate"""
         self.actionScoreGame.setText(
-            m18nc('@action:inmenu', "&Score Manual Game"))
+            i18nc('@action:inmenu', "&Score Manual Game"))
         self.actionScoreGame.setIconText(
-            m18nc('@action:intoolbar', 'Manual Game'))
-        self.actionScoreGame.setHelpText(
-            m18nc('kajongg @info:tooltip',
+            i18nc('@action:intoolbar', 'Manual Game'))
+        self.actionScoreGame.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   '&Score a manual game.'))
 
-        self.actionPlayGame.setText(m18nc('@action:intoolbar', "&Play"))
+        self.actionPlayGame.setText(i18nc('@action:intoolbar', "&Play"))
         self.actionPlayGame.setPriority(QAction.LowPriority)
-        self.actionPlayGame.setHelpText(
-            m18nc('kajongg @info:tooltip', 'Start a new game.'))
+        self.actionPlayGame.setWhatsThis(
+            i18nc('kajongg @info:tooltip', 'Start a new game.'))
 
-        self.actionAbortGame.setText(m18nc('@action:inmenu', "&Abort Game"))
+        self.actionAbortGame.setText(i18nc('@action:inmenu', "&Abort Game"))
         self.actionAbortGame.setPriority(QAction.LowPriority)
-        self.actionAbortGame.setHelpText(
-            m18nc('kajongg @info:tooltip',
+        self.actionAbortGame.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   'Abort the current game.'))
 
-        self.actionQuit.setText(m18nc('@action:inmenu', "&Quit Kajongg"))
+        self.actionQuit.setText(i18nc('@action:inmenu', "&Quit Kajongg"))
         self.actionQuit.setPriority(QAction.LowPriority)
 
-        self.actionPlayers.setText(m18nc('@action:intoolbar', "&Players"))
-        self.actionPlayers.setHelpText(
-            m18nc('kajongg @info:tooltip',
+        self.actionPlayers.setText(i18nc('@action:intoolbar', "&Players"))
+        self.actionPlayers.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   'define your players.'))
 
-        self.actionRulesets.setText(m18nc('@action:intoolbar', "&Rulesets"))
-        self.actionRulesets.setHelpText(
-            m18nc('kajongg @info:tooltip',
+        self.actionRulesets.setText(i18nc('@action:intoolbar', "&Rulesets"))
+        self.actionRulesets.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   'customize rulesets.'))
 
         self.actionAngle.setText(
-            m18nc('@action:inmenu',
+            i18nc('@action:inmenu',
                   "&Change Visual Angle"))
-        self.actionAngle.setIconText(m18nc('@action:intoolbar', "Angle"))
-        self.actionAngle.setHelpText(
-            m18nc('kajongg @info:tooltip',
+        self.actionAngle.setIconText(i18nc('@action:intoolbar', "Angle"))
+        self.actionAngle.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   "Change the visual appearance of the tiles."))
 
+        self.actionFullscreen.setText(
+            i18nc('@action:inmenu',
+                  "F&ull Screen Mode"))
+
         self.actionScoreTable.setText(
-            m18nc('kajongg @action:inmenu', "&Score Table"))
+            i18nc('kajongg @action:inmenu', "&Score Table"))
         self.actionScoreTable.setIconText(
-            m18nc('kajongg @action:intoolbar', "&Scores"))
-        self.actionScoreTable.setHelpText(m18nc('kajongg @info:tooltip',
-                                                "Show or hide the score table for the current game."))
+            i18nc('kajongg @action:intoolbar', "&Scores"))
+        self.actionScoreTable.setWhatsThis(i18nc('kajongg @info:tooltip',
+                                                 "Show or hide the score table for the current game."))
 
-        self.actionExplain.setText(m18nc('@action:inmenu', "&Explain Scores"))
-        self.actionExplain.setIconText(m18nc('@action:intoolbar', "&Explain"))
-        self.actionExplain.setHelpText(m18nc('kajongg @info:tooltip',
-                                             'Explain the scoring for all players in the current game.'))
+        self.actionExplain.setText(i18nc('@action:inmenu', "&Explain Scores"))
+        self.actionExplain.setIconText(i18nc('@action:intoolbar', "&Explain"))
+        self.actionExplain.setWhatsThis(i18nc('kajongg @info:tooltip',
+                                              'Explain the scoring for all players in the current game.'))
 
-        self.actionAutoPlay.setText(m18nc('@action:inmenu', "&Demo Mode"))
+        self.actionAutoPlay.setText(i18nc('@action:inmenu', "&Demo Mode"))
         self.actionAutoPlay.setPriority(QAction.LowPriority)
-        self.actionAutoPlay.setHelpText(m18nc('kajongg @info:tooltip',
-                                              'Let the computer take over for you. Start a new local game if needed.'))
+        self.actionAutoPlay.setWhatsThis(i18nc('kajongg @info:tooltip',
+                                               'Let the computer take over for you. Start a new local game if needed.'))
 
-        self.actionChat.setText(m18n("C&hat"))
-        self.actionChat.setHelpText(
-            m18nc('kajongg @info:tooltip',
+        self.actionChat.setText(i18n("C&hat"))
+        self.actionChat.setWhatsThis(
+            i18nc('kajongg @info:tooltip',
                   'Chat with the other players.'))
 
     def changeEvent(self, event):
@@ -571,14 +602,14 @@ class MainWindow(KXmlGuiWindow):
             self.rulesetWindow = RulesetSelector()
         self.rulesetWindow.show()
 
-    def adjustView(self):
+    def adjustMainView(self):
         """adjust the view such that exactly the wanted things are displayed
         without having to scroll"""
         if not Internal.scaleScene or not isAlive(self):
             return
         view, scene = self.centralView, self.scene
         if scene:
-            scene.adjustView()
+            scene.adjustSceneView()
             view.fitInView(scene.itemsBoundingRect(), Qt.KeepAspectRatio)
 
     @afterQueuedAnimations
@@ -595,10 +626,10 @@ class MainWindow(KXmlGuiWindow):
             self, dummyDeferredResult, dummyOldValue=None, dummyNewValue=None, *dummyArgs):
         """if the wanted tileset changed, apply the change now"""
         if self.centralView:
-            with MoveImmediate():
+            with AnimationSpeed():
                 if self.scene:
                     self.scene.applySettings()
-            self.adjustView()
+            self.adjustMainView()
 
     @afterQueuedAnimations
     def showSettings(self, dummyDeferredResult, dummyChecked=None):
@@ -617,12 +648,12 @@ class MainWindow(KXmlGuiWindow):
         """user has toggled widget visibility with an action"""
         assert self.scene
         action = self.sender()
-        actionData = variantValue(action.data())
+        actionData = action.data()
         if checked:
             if isinstance(actionData, type):
                 clsName = actionData.__name__
                 actionData = actionData(scene=self.scene)
-                action.setData(toQVariant(actionData))
+                action.setData(actionData)
                 setattr(
                     self.scene,
                     clsName[0].lower() + clsName[1:],
@@ -658,5 +689,5 @@ class MainWindow(KXmlGuiWindow):
     def changeAngle(self, deferredResult, dummyButtons=None, dummyModifiers=None): # pylint: disable=unused-argument
         """change the lightSource"""
         if self.scene:
-            with MoveImmediate():
+            with AnimationSpeed():
                 self.scene.changeAngle()
